==> bigdata-common-0.0.37-sources.jar: cn/com/duiba/bigdata/common/biz/utils/BigdataUtil.java <==
package cn.com.duiba.bigdata.common.biz.utils;

import cn.com.duiba.bigdata.common.biz.entity.redis.KeyIndex;
import cn.com.duiba.bigdata.common.biz.enums.DimStatisticsIndexEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.util.Bytes;
import org.joda.time.DateTime;

import java.math.BigDecimal;
import java.util.*;

@Slf4j
public class BigdataUtil {

    public static boolean numberCheck(String str) {
        return !StringUtils.isBlank(str) && str.matches("[0-9]*");
    }

    /**
     * 组装timekey
     *
     * @param key
     * @param timeList
     * @return
     */
    public static List<String> getTimeKeyList(String key, List<String> timeList) {

        List<String> keyList = new ArrayList<>();
        for (String time : timeList) {
            keyList.add(key + time);
        }
        return keyList;
    }

    //解析json数据
    public static JSONObject parseJson(Object content) {
        JSONObject jsonObject = null;
        if (content == null) {
            return null;
        }
        if (content instanceof String) {
            String json = (String) content;
            jsonObject = JSON.parseObject(json);
        } else if (content instanceof JSONObject) {
            jsonObject = (JSONObject) content;
        }
        return jsonObject;
    }

    public static String getString(JSONObject object, String key) {
        if (object == null) {
            return null;
        }
        String str = object.getString(key);
        if (StringUtils.isBlank(str)) {
            return null;
        }
        if ("null".equalsIgnoreCase(str)) {
            return null;
        }
        return str;
    }

    //获取hbase rowkey
    public static String getMD5HbaseRowkey(Object... obj) {
        String key = getJoinStr(obj);
        return StringUtils.isBlank(key) ? null : MD5Util.computeMD5(key).substring(0, 4) + "-" + key;
    }

    /**
     * 字符串拼接用下划线隔开
     *
     * @param obj
     * @return
     */
    public static String getJoinStr(Object... obj) {
        return getSepJoinStr("_", obj);
    }

    /**
     * 字符串拼接用下划线隔开
     *
     * @param obj
     * @return
     */
    public static String getSepJoinStr(String sep, Object... obj) {
        if (obj.length == 0) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (Object str : obj) {
            sb.append(str).append(sep);
        }
        return sb.delete(sb.length() - sep.length(), sb.length()).toString();
    }


    /**
     * 字符串拼接，没有下划线
     *
     * @param strs
     * @return
     */
    public static String getStr(String... strs) {
        StringBuilder sb = new StringBuilder();
        for (String str : strs) {
            sb.append(str);
        }
        return sb.toString();
    }

    /**
     * map数据融合 key field value
     *
     * @param map
     * @param rowkeyColumnMap
     */
    public static void mergeRowKeyMap(Map<String, String> map, Map<String, Map<String, Long>> rowkeyColumnMap) {
        for (String rowkey : map.keySet()) {
            JSONObject newJson = JSON.parseObject(map.get(rowkey));
            if (rowkeyColumnMap.containsKey(rowkey)) {
                Map<String, Long> oldColumnMap = rowkeyColumnMap.get(rowkey);
                mergeMap(newJson, oldColumnMap);
            } else {
                Map<String, Long> newColumnMap = new HashMap<>();
                for (String key : newJson.keySet()) {
                    Long value = newJson.getLongValue(key);
                    newColumnMap.put(key, value);
                }
                rowkeyColumnMap.put(rowkey, newColumnMap);
            }
        }
    }

    /**
     * 融合数据
     *
     * @param newJson
     * @param oldColumnMap
     */
    private static void mergeMap(JSONObject newJson, Map<String, Long> oldColumnMap) {
        for (String key : newJson.keySet()) {
            Long value = newJson.getLongValue(key);
            if (oldColumnMap.containsKey(key)) {
                oldColumnMap.put(key, oldColumnMap.get(key) + value);
            } else {
                oldColumnMap.put(key, value);
            }
        }
    }

    public static List<String> getTimeList(String type, String pattern, int number, Date date) {
        List<String> list = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        for (int i = 0; i < number; i++) {
            calendar.setTime(date);
            if ("day".equals(type)) {
                calendar.add(Calendar.DAY_OF_MONTH, -i);
            } else if ("hour".equals(type)) {
                calendar.add(Calendar.HOUR_OF_DAY, -i);
            } else if ("minute".equals(type)) {
                calendar.add(Calendar.MINUTE, -i);
            }
            list.add(DateFormatUtil.format(pattern, calendar.getTime()));
        }
        return list;
    }

    /**
     * 最近多少天/小时/分钟的时间列表
     *
     * @param pattern 时间格式
     * @param before  最近多久
     * @return
     */
    public static List<String> getTimeList(String pattern, int before) {
        DateTime dateTime = new DateTime();
        List<String> timeList = new ArrayList<>();
        for (int i = 0; i < before; i++) {
            if ("yyyyMMdd".equals(pattern)) {
                timeList.add(dateTime.plusDays(-i).toString(pattern));
            } else if ("yyyyMMddHH".equals(pattern)) {
                timeList.add(dateTime.plusHours(-i).toString(pattern));
            } else if ("yyyyMMddHHmm".equals(pattern)) {
                timeList.add(dateTime.plusMinutes(-i).toString(pattern));
            }
        }
        return timeList;
    }

    /**
     * 返回n分钟内的所有时间字符串
     *
     * @param before
     * @return
     */
    public static String[] getTimeArray(int before, String pattern) {
        DateTime dateTime = new DateTime();
        String[] times = new String[before];
        for (int i = 1; i <= before; i++) {
            times[i - 1] = dateTime.plusMinutes(-i).toString(pattern);
        }
        return times;
    }

    /**
     * 获取map中后端转化的数据
     *
     * @param data
     * @return
     */
    public static <T> Map<String, Long> getBackendCntMap(Map<String, T> data) {
        Map<String, Long> map = new HashMap<>();
        try {
            for (String key : data.keySet()) {
                if (numberCheck(key)) {
                    map.put(key, Long.valueOf(data.get(key).toString()));
                }
            }
        } catch (NumberFormatException e) {
            log.error("get backend data error", e);
        }
        return map;
    }

    /**
     * 除法-保留n位小数
     *
     * @param dividend 分子
     * @param divisor  分母
     * @param digit    保留小数位数
     * @return 结果字符串
     */
    public static <T extends Number> String division(T dividend, T divisor, int digit) {
        String result = "0";
        try {
            if (dividend != null && divisor != null) {
                double divisorDouble = divisor.doubleValue();
                double dividendDouble = dividend.doubleValue();
                if (divisorDouble != 0) {
                    result = new BigDecimal(dividendDouble / divisorDouble).setScale(digit, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
                }
            }
        } catch (Exception e) {
            log.error("division error", e);
        }
        return result;
    }

    /**
     * 获取实时数仓rediskey
     *
     * @param prefix 前缀
     * @param suffix 时间后缀
     * @param array  key维度
     * @return key
     */
    public static String getBigdataRedisKey(String prefix, String suffix, Object... array) {
        //redis前缀和后缀不为空
        if (StringUtils.isBlank(prefix)) {
            log.error("GetRedisKeyUDF prefix is null");
            return null;
        }

        //全局维度
        if (array == null || array.length == 0) {
            return prefix + "_" + suffix;
        }

        //array个数校验
        if (array.length % 2 != 0) {
            log.error("GetRedisKeyUDF eval, array.length % 2 != 0");
            return null;
        }

        List<KeyIndex> list = new ArrayList<>();
        for (int i = 0; i < array.length; i = i + 2) {
            String key = null;
            String value = null;
            if (array[i] != null) {
                key = array[i].toString();
            }
            if (array[i + 1] != null) {
                value = array[i + 1].toString();
            }
            if (StringUtils.isAnyBlank(key, value)) {
                return null;
            }
            Integer index = DimStatisticsIndexEnum.INSTANCE.getDimIndex(key);

            if (index == null) {
                log.error("key = {} is error, please check it.", key);
                return null;
            }
            KeyIndex keyIndex = new KeyIndex(key, value, index);
            list.add(keyIndex);
        }

        //根据dim index排序(升序)
        Collections.sort(list);

        //组装redis key
        StringBuilder sb = new StringBuilder();
        sb.append(prefix.toUpperCase()).append("_");
        for (KeyIndex info : list) {
            sb.append(info.getValue()).append("_");
        }
        if (StringUtils.isNotBlank(suffix)) {
            sb.append(suffix);
        }

        return sb.toString();
    }

    public static String getHbaseDataToString(Result result, String family, String column) {
        String value = "";
        if (result != null && result.containsColumn(family.getBytes(), column.getBytes())) {
            value = Bytes.toString(result.getValue(family.getBytes(), column.getBytes()));
        }
        return value;
    }

    public static String reverse(String str) {
        return new StringBuilder(str).reverse().toString();
    }

    public static Map<String, Map<String, String>> getHbaseStringDataMap(Map<String, List<String>> hbaseRowkeyMap, Result[] results, String family) {
        Map<String, Map<String, String>> map = new HashMap<>();
        if (MapUtils.isEmpty(hbaseRowkeyMap) || results == null) {
            return map;
        }
        for (Result result : results) {
            if (result != null && !result.isEmpty()) {
                Map<String, String> columnMap = new HashMap<>();
                String rowKey = Bytes.toString(result.getRow());
                List<String> columnList = hbaseRowkeyMap.get(rowKey);
                for (String column : columnList) {
                    putHbaseColumnData(columnMap, column, getHbaseDataToString(result, family, column));
                }
                map.put(rowKey, columnMap);
            }

        }
        return map;
    }


    public static void putHbaseColumnData(Map<String, String> map, String column, String value) {
        if (StringUtils.isNotBlank(value)) {
            map.put(column, value);
        }
    }

    /**
     * 解析url字段
     *
     * @param str
     * @param key
     * @return
     */
    public static String parseUrlParam(String str, String key) {
        if (StringUtils.isNotBlank(str)) {
            if (!str.startsWith(key)) {
                key = "&" + key;
            }
            if (str.contains(key)) {
                str = str.substring(str.indexOf(key) + key.length());
                return str.substring(0, str.contains("&") ? str.indexOf("&") : str.length());
            }
        }
        return null;
    }


    public static Map<String, Map<String, Long>> getHbaseLongDataMap(List<String> rowkeyList, Result[] results) {
        if (rowkeyList != null && rowkeyList.size() > 0) {
            Map<String, Map<String, Long>> map = new HashMap<>();
            for (int i = 0; i < rowkeyList.size(); i++) {
                Map<String, Long> columnMap = new HashMap<>();
                String rowkey = rowkeyList.get(i);
                Result result = results[i];
                if (result != null && !result.isEmpty()) {
                    Cell[] cells = result.rawCells();
                    for (Cell cell : cells) {
                        String column = Bytes.toString(cell.getQualifierArray(), cell.getQualifierOffset(), cell.getQualifierLength());
                        Long value = Bytes.toLong(cell.getValueArray(), cell.getValueOffset(), cell.getValueLength());
                        columnMap.put(column, value);
                    }
                    map.put(rowkey, columnMap);
                }
            }
            return map;
        }
        return null;
    }


    public static Map<String, Map<String, String>> getHbaseStringDataMap(List<String> rowkeyList, Result[] results) {
        if (rowkeyList != null && rowkeyList.size() > 0) {
            Map<String, Map<String, String>> map = new HashMap<>();
            for (int i = 0; i < rowkeyList.size(); i++) {
                Map<String, String> columnMap = new HashMap<>();
                String rowkey = rowkeyList.get(i);
                Result result = results[i];
                if (result != null && !result.isEmpty()) {
                    Cell[] cells = result.rawCells();
                    for (Cell cell : cells) {
                        String column = Bytes.toString(cell.getQualifierArray(), cell.getQualifierOffset(), cell.getQualifierLength());
                        String value = Bytes.toString(cell.getValueArray(), cell.getValueOffset(), cell.getValueLength());
                        putHbaseColumnData(columnMap, column, value);
                    }
                    map.put(rowkey, columnMap);
                }
            }
            return map;
        }
        return null;
    }

    /**
     * 求平均数
     *
     * @param set  数据集
     * @param skip 是否跳过0值
     * @param <T>  数据类型
     * @return 平均数
     */
    public static <T extends Number> Double avg(List<T> set, boolean skip) {
        int i = 0;
        double result = 0d;
        for (Number num : set) {
            if (num == null) {
                continue;
            }
            double value = num.doubleValue();

            if (skip && value == 0) {
                continue;
            }
            i = i + 1;
            result += value;
        }
        if (i != 0) {
            return result / i;
        }
        return 0d;

    }

    /**
     * 融合一批指标数据
     *
     * @param keyList
     * @param dataMap
     * @return
     */
    public static Map<String, Long> mergeData(List<String> keyList, Map<String, Map<String, String>> dataMap) {
        Map<String, Long> mergeMap = new HashMap<>();
        for (String key : keyList) {
            Map<String, String> data = dataMap.get(key);
            //取值
            for (String field : data.keySet()) {
                mergeMap.put(field, mergeMap.getOrDefault(field, 0L) + Long.valueOf(data.get(field)));
            }
        }

        return mergeMap;
    }

    /**
     * 合并redis或者hbase的结果数据
     *
     * @param resultMap 结果集合
     * @param <X>       key
     * @param <Y>       field
     * @return 合并后的集合
     */
    public static <X, Y> Map<X, Y> mergeResultMap(Map<X, Y>... resultMap) {
        if (resultMap == null || resultMap.length == 0) {
            return null;
        }
        Map<X, Y> dataMap = new HashMap<>();
        for (Map<X, Y> map : resultMap) {
            dataMap.putAll(map);
        }
        return dataMap;
    }


    public static String getHbaseJoinStr(Object... obj) {
        return getSepJoinStr("-", obj);
    }

}

==> bigdata-common-0.0.41-sources.jar: cn/com/duiba/bigdata/common/biz/utils/BigdataUtil.java <==
package cn.com.duiba.bigdata.common.biz.utils;

import cn.com.duiba.bigdata.common.biz.entity.redis.KeyIndex;
import cn.com.duiba.bigdata.common.biz.enums.DimStatisticsIndexEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.CellUtil;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.hadoop.hbase.util.CollectionUtils;
import org.joda.time.DateTime;

import java.math.BigDecimal;
import java.util.*;

@Slf4j
public class BigdataUtil {

    public static boolean numberCheck(String str) {
        return !StringUtils.isBlank(str) && str.matches("[0-9]*");
    }

    /**
     * 组装timekey
     *
     * @param key
     * @param timeList
     * @return
     */
    public static List<String> getTimeKeyList(String key, List<String> timeList) {

        List<String> keyList = new ArrayList<>();
        for (String time : timeList) {
            keyList.add(key + time);
        }
        return keyList;
    }

    //解析json数据
    public static JSONObject parseJson(Object content) {
        JSONObject jsonObject = null;
        if (content == null) {
            return null;
        }
        if (content instanceof String) {
            String json = (String) content;
            jsonObject = JSON.parseObject(json);
        } else if (content instanceof JSONObject) {
            jsonObject = (JSONObject) content;
        }
        return jsonObject;
    }

    public static String getString(JSONObject object, String key) {
        if (object == null) {
            return null;
        }
        String str = object.getString(key);
        if (StringUtils.isBlank(str)) {
            return null;
        }
        if ("null".equalsIgnoreCase(str)) {
            return null;
        }
        return str;
    }

    //获取hbase rowkey
    public static String getMD5HbaseRowkey(Object... obj) {
        String key = getJoinStr(obj);
        return StringUtils.isBlank(key) ? null : MD5Util.computeMD5(key).substring(0, 4) + "-" + key;
    }

    /**
     * 字符串拼接用下划线隔开
     *
     * @param obj
     * @return
     */
    public static String getJoinStr(Object... obj) {
        return getSepJoinStr("_", obj);
    }

    /**
     * 字符串拼接用下划线隔开
     *
     * @param obj
     * @return
     */
    public static String getSepJoinStr(String sep, Object... obj) {
        if (obj.length == 0) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (Object str : obj) {
            sb.append(str).append(sep);
        }
        return sb.delete(sb.length() - sep.length(), sb.length()).toString();
    }


    /**
     * 字符串拼接，没有下划线
     *
     * @param strs
     * @return
     */
    public static String getStr(String... strs) {
        StringBuilder sb = new StringBuilder();
        for (String str : strs) {
            sb.append(str);
        }
        return sb.toString();
    }

    /**
     * map数据融合 key field value
     *
     * @param map
     * @param rowkeyColumnMap
     */
    public static void mergeRowKeyMap(Map<String, String> map, Map<String, Map<String, Long>> rowkeyColumnMap) {
        for (String rowkey : map.keySet()) {
            JSONObject newJson = JSON.parseObject(map.get(rowkey));
            if (rowkeyColumnMap.containsKey(rowkey)) {
                Map<String, Long> oldColumnMap = rowkeyColumnMap.get(rowkey);
                mergeMap(newJson, oldColumnMap);
            } else {
                Map<String, Long> newColumnMap = new HashMap<>();
                for (String key : newJson.keySet()) {
                    Long value = newJson.getLongValue(key);
                    newColumnMap.put(key, value);
                }
                rowkeyColumnMap.put(rowkey, newColumnMap);
            }
        }
    }

    /**
     * 融合数据
     *
     * @param newJson
     * @param oldColumnMap
     */
    private static void mergeMap(JSONObject newJson, Map<String, Long> oldColumnMap) {
        for (String key : newJson.keySet()) {
            Long value = newJson.getLongValue(key);
            if (oldColumnMap.containsKey(key)) {
                oldColumnMap.put(key, oldColumnMap.get(key) + value);
            } else {
                oldColumnMap.put(key, value);
            }
        }
    }

    public static List<String> getTimeList(String type, String pattern, int number, Date date) {
        List<String> list = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        for (int i = 0; i < number; i++) {
            calendar.setTime(date);
            if ("day".equals(type)) {
                calendar.add(Calendar.DAY_OF_MONTH, -i);
            } else if ("hour".equals(type)) {
                calendar.add(Calendar.HOUR_OF_DAY, -i);
            } else if ("minute".equals(type)) {
                calendar.add(Calendar.MINUTE, -i);
            }
            list.add(DateFormatUtil.format(pattern, calendar.getTime()));
        }
        return list;
    }

    /**
     * 最近多少天/小时/分钟的时间列表
     *
     * @param pattern 时间格式
     * @param before  最近多久
     * @return
     */
    public static List<String> getTimeList(String pattern, int before) {
        DateTime dateTime = new DateTime();
        List<String> timeList = new ArrayList<>();
        for (int i = 0; i < before; i++) {
            if ("yyyyMMdd".equals(pattern)) {
                timeList.add(dateTime.plusDays(-i).toString(pattern));
            } else if ("yyyyMMddHH".equals(pattern)) {
                timeList.add(dateTime.plusHours(-i).toString(pattern));
            } else if ("yyyyMMddHHmm".equals(pattern)) {
                timeList.add(dateTime.plusMinutes(-i).toString(pattern));
            }
        }
        return timeList;
    }

    /**
     * 返回n分钟内的所有时间字符串
     *
     * @param before
     * @return
     */
    public static String[] getTimeArray(int before, String pattern) {
        DateTime dateTime = new DateTime();
        String[] times = new String[before];
        for (int i = 1; i <= before; i++) {
            times[i - 1] = dateTime.plusMinutes(-i).toString(pattern);
        }
        return times;
    }

    /**
     * 获取map中后端转化的数据
     *
     * @param data
     * @return
     */
    public static <T> Map<String, Long> getBackendCntMap(Map<String, T> data) {
        Map<String, Long> map = new HashMap<>();
        try {
            for (String key : data.keySet()) {
                if (numberCheck(key)) {
                    map.put(key, Long.valueOf(data.get(key).toString()));
                }
            }
        } catch (NumberFormatException e) {
            log.error("get backend data error", e);
        }
        return map;
    }

    /**
     * 除法-保留n位小数
     *
     * @param dividend 分子
     * @param divisor  分母
     * @param digit    保留小数位数
     * @return 结果字符串
     */
    public static <T extends Number> String division(T dividend, T divisor, int digit) {
        String result = "0";
        try {
            if (dividend != null && divisor != null) {
                double divisorDouble = divisor.doubleValue();
                double dividendDouble = dividend.doubleValue();
                if (divisorDouble != 0) {
                    result = new BigDecimal(dividendDouble / divisorDouble).setScale(digit, BigDecimal.ROUND_HALF_UP).stripTrailingZeros().toPlainString();
                }
            }
        } catch (Exception e) {
            log.error("division error", e);
        }
        return result;
    }

    /**
     * 获取实时数仓rediskey
     *
     * @param prefix 前缀
     * @param suffix 时间后缀
     * @param array  key维度
     * @return key
     */
    public static String getBigdataRedisKey(String prefix, String suffix, Object... array) {
        //redis前缀和后缀不为空
        if (StringUtils.isBlank(prefix)) {
            log.error("GetRedisKeyUDF prefix is null");
            return null;
        }

        //全局维度
        if (array == null || array.length == 0) {
            return prefix + "_" + suffix;
        }

        //array个数校验
        if (array.length % 2 != 0) {
            log.error("GetRedisKeyUDF eval, array.length % 2 != 0");
            return null;
        }

        List<KeyIndex> list = new ArrayList<>();
        for (int i = 0; i < array.length; i = i + 2) {
            String key = null;
            String value = null;
            if (array[i] != null) {
                key = array[i].toString();
            }
            if (array[i + 1] != null) {
                value = array[i + 1].toString();
            }
            if (StringUtils.isAnyBlank(key, value)) {
                return null;
            }
            Integer index = DimStatisticsIndexEnum.INSTANCE.getDimIndex(key);

            if (index == null) {
                log.error("key = {} is error, please check it.", key);
                return null;
            }
            KeyIndex keyIndex = new KeyIndex(key, value, index);
            list.add(keyIndex);
        }

        //根据dim index排序(升序)
        Collections.sort(list);

        //组装redis key
        StringBuilder sb = new StringBuilder();
        sb.append(prefix.toUpperCase()).append("_");
        for (KeyIndex info : list) {
            sb.append(info.getValue()).append("_");
        }
        if (StringUtils.isNotBlank(suffix)) {
            sb.append(suffix);
        }

        return sb.toString();
    }

    public static String getHbaseDataToString(Result result, String family, String column) {
        String value = "";
        if (result != null && result.containsColumn(family.getBytes(), column.getBytes())) {
            value = Bytes.toString(result.getValue(family.getBytes(), column.getBytes()));
        }
        return value;
    }

    public static String reverse(String str) {
        return new StringBuilder(str).reverse().toString();
    }

    public static Map<String, Map<String, String>> getHbaseStringDataMap(Map<String, List<String>> hbaseRowkeyMap, Result[] results, String family) {
        Map<String, Map<String, String>> map = new HashMap<>();
        if (MapUtils.isEmpty(hbaseRowkeyMap) || results == null) {
            return map;
        }
        for (Result result : results) {
            if (result != null && !result.isEmpty()) {
                Map<String, String> columnMap = new HashMap<>();
                String rowKey = Bytes.toString(result.getRow());
                List<String> columnList = hbaseRowkeyMap.get(rowKey);
                for (String column : columnList) {
                    putHbaseColumnData(columnMap, column, getHbaseDataToString(result, family, column));
                }
                map.put(rowKey, columnMap);
            }

        }
        return map;
    }


    public static void putHbaseColumnData(Map<String, String> map, String column, String value) {
        if (StringUtils.isNotBlank(value)) {
            map.put(column, value);
        }
    }

    /**
     * 解析url字段
     *
     * @param str
     * @param key
     * @return
     */
    public static String parseUrlParam(String str, String key) {
        if (StringUtils.isNotBlank(str)) {
            if (!str.startsWith(key)) {
                key = "&" + key;
            }
            if (str.contains(key)) {
                str = str.substring(str.indexOf(key) + key.length());
                return str.substring(0, str.contains("&") ? str.indexOf("&") : str.length());
            }
        }
        return null;
    }


    public static Map<String, Map<String, Long>> getHbaseLongDataMap(List<String> rowkeyList, Result[] results) {
        if (rowkeyList != null && rowkeyList.size() > 0) {
            Map<String, Map<String, Long>> map = new HashMap<>();
            for (int i = 0; i < rowkeyList.size(); i++) {
                Map<String, Long> columnMap = new HashMap<>();
                String rowkey = rowkeyList.get(i);
                Result result = results[i];
                if (result != null && !result.isEmpty()) {
                    Cell[] cells = result.rawCells();
                    for (Cell cell : cells) {
                        String column = Bytes.toString(cell.getQualifierArray(), cell.getQualifierOffset(), cell.getQualifierLength());
                        Long value = Bytes.toLong(cell.getValueArray(), cell.getValueOffset(), cell.getValueLength());
                        columnMap.put(column, value);
                    }
                    map.put(rowkey, columnMap);
                }
            }
            return map;
        }
        return null;
    }

    public static Map<String, String> getHbaseStringDataMap(Result result, String family) {
        if (result == null || result.isEmpty()) {
            return null;
        }
        List<Cell> cellList = result.listCells();
        if (CollectionUtils.isEmpty(cellList)) {
            return null;
        }
        Map<String, String> map = new HashMap<>();
        for (Cell cell : cellList) {
            if (!Bytes.toString(CellUtil.cloneFamily(cell)).equalsIgnoreCase(family)) {
                continue;
            }
            String column = Bytes.toString(CellUtil.cloneQualifier(cell));
            String value = Bytes.toString(CellUtil.cloneValue(cell));
            map.put(column, value);
        }
        return map;
    }

    public static Map<String, Map<String, String>> getHbaseStringDataMap(List<String> rowkeyList, Result[] results) {
        if (rowkeyList != null && rowkeyList.size() > 0) {
            Map<String, Map<String, String>> map = new HashMap<>();
            for (int i = 0; i < rowkeyList.size(); i++) {
                Map<String, String> columnMap = new HashMap<>();
                String rowkey = rowkeyList.get(i);
                Result result = results[i];
                if (result != null && !result.isEmpty()) {
                    Cell[] cells = result.rawCells();
                    for (Cell cell : cells) {
                        String column = Bytes.toString(cell.getQualifierArray(), cell.getQualifierOffset(), cell.getQualifierLength());
                        String value = Bytes.toString(cell.getValueArray(), cell.getValueOffset(), cell.getValueLength());
                        putHbaseColumnData(columnMap, column, value);
                    }
                    map.put(rowkey, columnMap);
                }
            }
            return map;
        }
        return null;
    }

    /**
     * 求平均数
     *
     * @param set  数据集
     * @param skip 是否跳过0值
     * @param <T>  数据类型
     * @return 平均数
     */
    public static <T extends Number> Double avg(List<T> set, boolean skip) {
        int i = 0;
        double result = 0d;
        for (Number num : set) {
            if (num == null) {
                continue;
            }
            double value = num.doubleValue();

            if (skip && value == 0) {
                continue;
            }
            i = i + 1;
            result += value;
        }
        if (i != 0) {
            return result / i;
        }
        return 0d;

    }

    /**
     * 融合一批指标数据
     *
     * @param keyList
     * @param dataMap
     * @return
     */
    public static Map<String, Long> mergeData(List<String> keyList, Map<String, Map<String, String>> dataMap) {
        Map<String, Long> mergeMap = new HashMap<>();
        for (String key : keyList) {
            Map<String, String> data = dataMap.get(key);
            //取值
            for (String field : data.keySet()) {
                mergeMap.put(field, mergeMap.getOrDefault(field, 0L) + Long.valueOf(data.get(field)));
            }
        }

        return mergeMap;
    }

    /**
     * 合并redis或者hbase的结果数据
     *
     * @param resultMap 结果集合
     * @param <X>       key
     * @param <Y>       field
     * @return 合并后的集合
     */
    public static <X, Y> Map<X, Y> mergeResultMap(Map<X, Y>... resultMap) {
        if (resultMap == null || resultMap.length == 0) {
            return null;
        }
        Map<X, Y> dataMap = new HashMap<>();
        for (Map<X, Y> map : resultMap) {
            dataMap.putAll(map);
        }
        return dataMap;
    }


    public static String getHbaseJoinStr(Object... obj) {
        return getSepJoinStr("-", obj);
    }

}
